import { Selector, createSelector } from '@ngrx/store';
import { EntityState, createEntityAdapter } from '@ngrx/entity';
import { GroupDetailData } from '@ucap/protocol-sync';

export interface GroupState extends EntityState<GroupDetailData> {
  syncDate: string;
}
export const adapterGroup = createEntityAdapter<GroupDetailData>({
  selectId: groupDetailData => groupDetailData.seq
});

export interface State {
  groups: GroupState;
}

const groupInitialState: GroupState = adapterGroup.getInitialState({
  syncDate: ''
});

export const initialState: State = {
  groups: groupInitialState
};

const {
  selectAll: selectAllForGroup,
  selectEntities: selectEntitiesForGroup,
  selectIds: selectIdsForGroup,
  selectTotal: selectTotalForGroup
} = adapterGroup.getSelectors();

export function selectors<S>(selector: Selector<any, State>) {
  const selectGroups = createSelector(selector, (state: State) => state.groups);

  return {
    groups: createSelector(selectGroups, selectAllForGroup),
    groupSyncDate: createSelector(
      selectGroups,
      groupState => groupState.syncDate
    )
  };
}
