import { of } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import { QueryProtocolService } from '@ucap/ng-protocol-query';

import {
  dept,
  deptSuccess,
  deptFailure,
  myDeptUserSuccess,
  myDeptUser,
  myDeptUserFailure
} from './actions';

@Injectable()
export class Effects {
  dept$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(dept),
      map(action => action.req),
      switchMap(req => {
        return this.queryProtocolService.dept(req).pipe(
          map(res => {
            const departmentInfoList = res.departmentInfoList.sort((a, b) =>
              a.order < b.order ? -1 : a.order > b.order ? 1 : 0
            );

            return deptSuccess({ departmentInfoList });
          }),
          catchError(error => of(deptFailure({ error })))
        );
      })
    );
  });

  myDeptUser$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(myDeptUser),
      map(action => action.req),
      switchMap(req => {
        return this.queryProtocolService.deptUser(req).pipe(
          map(res => {
            const userInfos = res.userInfos.sort((a, b) =>
              a.order < b.order
                ? -1
                : a.order > b.order
                ? 1
                : a.name < b.name
                ? -1
                : a.name > b.name
                ? 1
                : 0
            );
            return myDeptUserSuccess({ userInfos });
          }),
          catchError(error => of(myDeptUserFailure({ error })))
        );
      })
    );
  });

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private queryProtocolService: QueryProtocolService
  ) {}
}
